/*

Copyright (c) 2009, Oliver Martin <oliver@volatilevoid.net>

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

#define CONFIG_SENSOR_M4MO
#define FLASH_CMD       200
#define FLASH_CAMERA    0
#define FLASH_MOVIE     1
#define FLASH_CMD_ON    1
#define FLASH_CMD_OFF   0

  
#include "net_volatilevoid_glowingpear_Lightbulb.h"
#include <sys/ioctl.h>
#include <media/msm_camera.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

int fd;

JNIEXPORT jboolean JNICALL Java_net_volatilevoid_glowingpear_Lightbulb_open
  (JNIEnv *env, jobject obj, jstring dev)
{
	jboolean iscopy;
	
	const char *cdev = (*env)->GetStringUTFChars(env, dev, &iscopy);
	if (!cdev)
		return JNI_FALSE;
	
	fd = open(cdev, O_WRONLY);
	if (fd == -1)
		return JNI_FALSE;
	
	return JNI_TRUE;
}

JNIEXPORT jboolean JNICALL Java_net_volatilevoid_glowingpear_Lightbulb_close
  (JNIEnv *env, jobject obj)
{
	if (0 == close(fd))
		return JNI_TRUE;
	else
		return JNI_FALSE;
}

jboolean set_led(int type, int state)
{
	ioctl_msg_info info;
	
	info.codeA = FLASH_CMD;
	info.codeB = type;
	info.codeC = state;
	
	if (0 == ioctl(fd, MSM_CAM_IOCTL_PGH_MSG, &info))
		return JNI_TRUE;
	else
		return JNI_FALSE;
}

JNIEXPORT jboolean JNICALL Java_net_volatilevoid_glowingpear_Lightbulb_start
  (JNIEnv *env, jobject obj)
{
	return set_led(FLASH_MOVIE, FLASH_CMD_ON);
}

JNIEXPORT jboolean JNICALL Java_net_volatilevoid_glowingpear_Lightbulb_stop
  (JNIEnv *env, jobject obj)
{
	return set_led(FLASH_MOVIE, FLASH_CMD_OFF);
}

JNIEXPORT jboolean JNICALL Java_net_volatilevoid_glowingpear_Lightbulb_flash
  (JNIEnv *env, jobject obj, jint usecs)
{
	set_led(FLASH_CAMERA, FLASH_CMD_ON);
	usleep(usecs);
	return set_led(FLASH_MOVIE, FLASH_CMD_ON);
}
